#!/usr/bin/env python
# -*- coding: utf-8 -*-
# import psutil
import urllib2
import time
import plugins


class Plugin(plugins.BasePlugin):
    __name__ = 'nginx'

    def run(self, config):
        '''
        Provides the following metrics (example):
            "accepts": 588462,
            "accepts_per_second": 0.0,
            "active_connections": 192,
            "handled": 588462,
            "handled_per_second": 0.0,
            "reading": 0,
            "requests": 9637106,
            "requests_per_second": 0.0,
            "waiting": 189,
            "writing": 3

        requests, accepts, handled are values since the start of nginx.
        *_per_second values calculated from them using cached values from previous call.
        '''

        try:
            results = dict()
            next_cache = dict()
            request = urllib2.Request(config.get('nginx', 'status_page_url'))
            raw_response = urllib2.urlopen(request)
            next_cache['ts'] = time.time()
            prev_cache = self.get_agent_cache()  # Get absolute values from previous check
            response = raw_response.readlines()

            # Active connections: N
            # active_connections = response[0].split(':')[1].strip()
            active_connections = response[0].split()[-1]
            results['active_connections'] = int(active_connections)

            # server accepts handled requests
            keys = response[1].split()[1:]
            values = response[2].split()
            for key, value in zip(keys, values):
                next_cache[key] = int(value)
                results[key] = next_cache[key]  # Keep absolute values in results
                try:
                    if next_cache[key] >= prev_cache[key]:
                        results['%s_per_second' % key] = \
                            (next_cache[key] - prev_cache[key]) / \
                            (next_cache['ts'] - prev_cache['ts'])
                    else:  # Nginx was restarted after previous caching
                        results['%s_per_second' % key] = \
                            next_cache[key] / \
                            (next_cache['ts'] - prev_cache['ts'])
                except KeyError:  # No cache yet, can't calculate
                    results['%s_per_second' % key] = 0.0

            # Reading: X Writing: Y Waiting: Z
            keys = response[3].split()[0::2]
            keys = [entry.strip(':').lower() for entry in keys]
            values = response[3].split()[1::2]
            for key, value in zip(keys, values):
                results[key] = int(value)

            # Cache absolute values for next check calculations
            self.set_agent_cache(next_cache)

            return results
        except Exception:
            return False


if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re
from subprocess import Popen, PIPE, CalledProcessError
import sys
import plugins


def _get_match_groups(ping_output, regex):
    match = regex.search(ping_output)
    if not match:
        return False
    else:
        return match.groups()


def system_command(Command, newlines=True):
    Output = ""
    Error = ""
    try:
        proc = Popen(Command.split(), stdout=PIPE)
        Output = proc.communicate()[0]
    except Exception:
        pass

    if Output:
        if newlines is True:
            Stdout = Output.split("\\n")
        else:
            Stdout = Output
    else:
        Stdout = []
    if Error:
        Stderr = Error.split("\n")
    else:
        Stderr = []

    return (Stdout, Stderr)


def collect_ping(hostname):
    if sys.platform.startswith('linux') or sys.platform.startswith('freebsd'):
    #if sys.platform == "linux" or sys.platform == "linux2":
        response = str(system_command("ping -W 5 -c 1 " + hostname, False)[0])
        try:
            matcher = re.compile(r'(\d+.\d+)/(\d+.\d+)/(\d+.\d+)/(\d+.\d+)')
            minping, avgping, maxping, jitter = _get_match_groups(response, matcher)
            response = avgping
        except Exception:
            #response = 9999
            response = -1
    elif sys.platform == "darwin":
        response = str(system_command("ping -c 1 " + hostname, False)[0])
        # matcher = re.compile(r'min/avg/max/stddev = (\d+)/(\d+)/(\d+)/(\d+) ms')
        # min, avg, max, stddev = _get_match_groups(response, matcher)
        matcher = re.compile(r'(\d+.\d+)/(\d+.\d+)/(\d+.\d+)/(\d+.\d+)')
        matched = _get_match_groups(response, matcher)
        if matched is False:
            #response = 0
            response = -1
        else:
            minping, avgping, maxping, jitter = matched
            response = avgping
    elif sys.platform == "win32":
        #response = 0
        response = -1
        try:
            ping = Popen(["ping", "-n", "1 ", hostname], stdout=PIPE, stderr=PIPE)
            out, error = ping.communicate()
            if out:
                try:
                    response = int(re.findall(r"Average = (\d+)", out)[0])
                except Exception:
                    pass
            else:
                #response = 0
                response = -1
        except CalledProcessError:
            pass
    else:
        #response = float(system_command("ping -W -c 1 " + hostname))
        response = -1
    return {'avgping': response, 'host': hostname}


class Plugin(plugins.BasePlugin):
    __name__ = 'ping'

    def run(self, config):
        data = {}
        my_hosts = config.get('ping', 'hosts').split(',')
        data['ping'] = []
        for host in my_hosts:
            data['ping'].append(collect_ping(host))
        return data['ping']


if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import time
import plugins


class Plugin(plugins.BasePlugin):
    __name__ = 'sleeper'

    def run(self, *unused):
        time.sleep(60 * 60 * 24)


if __name__ == '__main__':
    Plugin().run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import psutil
import plugins


class Plugin(plugins.BasePlugin):
    __name__ = 'cpu'

    def run(self, *unused):
        results = {}
        data = psutil.cpu_times_percent(interval=1, percpu=True)
        cpu_number = -1
        for cpu in data:
            core = {}
            cpu_number = cpu_number+1
            results[cpu_number] = {}
            for key in cpu._fields:
                core[key] = getattr(cpu, key)
            results[cpu_number] = core
        return results


if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import plugins
import os
import time
import re
import urllib2
import base64

class Plugin(plugins.BasePlugin):
    __name__ = 'litespeed'

    '''
    Litespeed monitoring plugin. Add the following section to /etc/nixstats.ini
    
    [litespeed]
    enabled=yes
    host=localhost
    port=7080
    username=admin
    password=pass
    '''

    def run(self, config):
        result = {}
        results = {}
        data = False
        prev_cache = self.get_agent_cache()  # Get absolute values from previous check
        request = urllib2.Request("http://%s:%s/status?rpt=summary" % (config.get('litespeed', 'host'),config.get('litespeed', 'port')))
        base64string = base64.b64encode('%s:%s' % (config.get('litespeed', 'username'), config.get('litespeed', 'password')))
        request.add_header("Authorization", "Basic %s" % base64string)
        response = urllib2.urlopen(request).read()

        for line in response.split('\n'):
            test = re.search('REQ_RATE \[(.*)\]', line)
            if test is not None and test.group(1):
                data = True
                try:
                    result[test.group(1)]
                except KeyError:
                    result[test.group(1)] = {}
                lines = line.replace('\n', '').replace(test.group(0), '').split(', ')
                for line in lines:
                    keyval = line.strip(':').strip().split(':')
                    try:
                        result[test.group(1)][keyval[0]] += float(keyval[1])
                    except KeyError:
                        result[test.group(1)][keyval[0]] = float(keyval[1])

        metrics = (
                'SSL_BPS_IN',
                'BPS_OUT',
                'MAXSSL_CONN',
                'PLAINCONN',
                'BPS_IN',
                'SSLCONN',
                'AVAILSSL',
                'IDLECONN',
                'SSL_BPS_OUT',
                'AVAILCONN',
                'MAXCONN',
                'REQ_PROCESSING'
        )

        if data is True:
            for vhost, statistics in result.items():
                try:
                    prev_cache[vhost]['ts'] = prev_cache['ts']
                except KeyError:
                    prev_cache[vhost] = {}
                results[vhost] = {}
                for key, value in statistics.items():
                    if key == 'TOT_REQS':
                        results[vhost]['RPS'] = self.absolute_to_per_second(key, value, prev_cache[vhost])
                    if key == 'TOTAL_STATIC_HITS':
                        results[vhost]['STATIC_RPS'] = self.absolute_to_per_second(key, value, prev_cache[vhost])
                    if key == 'TOTAL_PUB_CACHE_HITS':
                        results[vhost]['PUB_CACHE_RPS'] = self.absolute_to_per_second(key, value, prev_cache[vhost])
                    if key == 'TOTAL_PRIVATE_CACHE_HITS':
                        results[vhost]['PRIVATE_CACHE_RPS'] = self.absolute_to_per_second(key, value, prev_cache[vhost])
                    if key in metrics:
                        results[vhost][key] = value

        result['ts'] = time.time()
        self.set_agent_cache(result)
        return results

if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import psutil
import plugins


class Plugin(plugins.BasePlugin):
    __name__ = 'diskusage'

    def run(self, config):
        disk = {}
        disk['df-psutil'] = []
        for part in psutil.disk_partitions(False):
            if os.name == 'nt':
                if 'cdrom' in part.opts or part.fstype == '':
                    # skip cd-rom drives with no disk in it; they may raise
                    # ENOENT, pop-up a Windows GUI error for a non-ready
                    # partition or just hang.
                    continue
            try:
                usage = psutil.disk_usage(part.mountpoint)
                diskdata = {}
                diskdata['info'] = part
                for key in usage._fields:
                    diskdata[key] = getattr(usage, key)
                disk['df-psutil'].append(diskdata)
            except:
                pass

        try:
                force_df = config.get('diskusage', 'force_df')
        except:
                force_df = 'no'

        if len(disk['df-psutil']) == 0 or force_df == 'yes': 
            try:
	        disk['df-psutil'] = []
                df_output_lines = [s.split() for s in os.popen("df -Pl").read().splitlines()] 
                del df_output_lines[0]
                for row in df_output_lines:
                    if row[0] == 'tmpfs':
                        continue
                    disk['df-psutil'].append({'info': [row[0], row[5],'',''], 'total': int(row[1])*1024, 'used': int(row[2])*1024, 'free': int(row[3])*1024, 'percent': row[4][:-1]}) 
            except:
                pass

        return disk


if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python

import urllib2
import time
import plugins
from pymongo import MongoClient

class Plugin(plugins.BasePlugin):
    __name__ = 'mongodb'

    def run(self, config):
        """
        Mongodb monitoring
        """

        client = MongoClient(config.get('mongodb', 'connection_string'))

        db = client.admin

        statistics = db.command("serverStatus")
        prev_cache = self.get_agent_cache()  # Get absolute values from previous check

        data = {}
        results = dict()
        data['connections.totalCreated'] = statistics['connections']['totalCreated']

        results['connections.available'] = statistics['connections']['available']
        results['connections.current'] = statistics['connections']['current']

        data['opcounters.command'] = statistics['opcounters']['command']
        data['opcounters.delete'] = statistics['opcounters']['delete']
        data['opcounters.getmore'] = statistics['opcounters']['getmore']
        data['opcounters.insert'] = statistics['opcounters']['insert']
        data['opcounters.query'] = statistics['opcounters']['query']
        data['opcounters.update'] = statistics['opcounters']['update']

        data['opLatencies.commands.latency'] = statistics['opLatencies']['commands']['latency']
        data['opLatencies.commands.ops'] = statistics['opLatencies']['commands']['ops']
        data['opLatencies.reads.latency'] = statistics['opLatencies']['reads']['latency']
        data['opLatencies.reads.ops'] = statistics['opLatencies']['reads']['ops']
        data['opLatencies.writes.latency'] = statistics['opLatencies']['writes']['latency']
        data['opLatencies.writes.ops'] = statistics['opLatencies']['writes']['ops']

        data['asserts.msg'] = statistics['asserts']['msg']
        data['asserts.regular'] = statistics['asserts']['regular']
        data['asserts.rollovers'] = statistics['asserts']['rollovers']
        data['asserts.user'] = statistics['asserts']['user']
        data['asserts.warning'] = statistics['asserts']['warning']
 
        try:
            results['repl']['hosts'] = statistics['repl']['hosts']
            results['repl']['isMaster'] = statistics['repl']['isMaster']
            results['repl']['secondary'] = statistics['repl']['secondary']
            results['repl']['setName'] = statistics['repl']['setName']

            data['opcountersRepl.command'] = statistics['opcountersRepl']['command']
            data['opcountersRepl.delete'] = statistics['opcountersRepl']['delete']
            data['opcountersRepl.getmore'] = statistics['opcountersRepl']['getmore']
            data['opcountersRepl.insert'] = statistics['opcountersRepl']['insert']
            data['opcountersRepl.query'] = statistics['opcountersRepl']['query']
            data['opcountersRepl.update'] = statistics['opcountersRepl']['update']
        except KeyError:
            pass

        for key, val in data.items():
            results[key] = self.absolute_to_per_second(key, val, prev_cache)

        try:
            results['opLatencies.commands'] = results['opLatencies.commands.latency']/results['opLatencies.commands.ops']
            results['opLatencies.writes'] = results['opLatencies.writes.latency']/results['opLatencies.writes.ops']
            results['opLatencies.reads'] = results['opLatencies.reads.latency']/results['opLatencies.reads.ops']
        except:
            pass
            
        next_cache = data
        next_cache['ts'] = time.time()
        self.set_agent_cache(next_cache)
        results['mem'] = statistics['mem']

        return results


if __name__ == '__main__':
    Plugin().execute()
